/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
package com.ibm.rational.test.lm.spi.driver.test;

import java.net.URI;
import java.util.HashMap;

import junit.framework.TestCase;

import com.ibm.ccl.soa.deploy.core.Topology;
import com.ibm.ccl.soa.deploy.server.ServerUnit;
import com.ibm.rational.test.lm.spi.common.IRestAPIResponseCodes;
import com.ibm.rational.test.lm.spi.common.IRestResponse;
import com.ibm.rational.test.lm.spi.driver.test.util.ProviderXml;
import com.ibm.rational.test.lm.spi.driver.test.util.Registry;
import com.ibm.rational.test.lm.spi.driver.test.util.TestTracer;
import com.ibm.rational.test.lm.spi.driver.test.util.TopologyUtils;
import com.ibm.rational.test.lm.spi.driver.test.util.ProviderXml.ProviderServerInfo;

public class Inventory extends TestCase 
{
    private static String inventoryHostname = "";    
    private static String inventoryPortNumber = "";
    
    private static final String SINGLE_SLASH = "/";
	private static final String COLON = ":";
    
    private static final String assetUrlPart = "/asset/";
    private static final String discoveryControlUrlPart = "/discoveryControl/";
    private static final String assetStatusUrlPart = "/assetStatus/";
    
    private static String baseAssetUrl;
    private static String baseDiscoveryControlUrl;
    private static String baseAssetStatusUrl;
    
    private static HashMap<String, String> providerToConfig = new HashMap<String, String>();
    
    private static boolean isInited = false;
    
    protected void setUp() throws Exception
    {
        assertNotNull(Registry.getRegistry().getInventoryProvider());
        if (!isInited)
        {
            assertTrue(initSPIData());
            isInited = true;
        }
    }
    
    private boolean initSPIData()
    {
        boolean success = false;
        providerToConfig = ProviderXml.getProviderDocs();
        success = providerToConfig.size() > 0;
        ProviderServerInfo serverInfo = 
            ProviderXml.getProviderServerInfo(providerToConfig.get(Registry.getRegistry().getInventoryProviderPluginId()));
        inventoryHostname = serverInfo.hostname;
        inventoryPortNumber = serverInfo.port;
        baseAssetUrl = SINGLE_SLASH + inventoryHostname + COLON + inventoryPortNumber + assetUrlPart;
        baseDiscoveryControlUrl = SINGLE_SLASH + inventoryHostname + COLON + inventoryPortNumber + discoveryControlUrlPart;
        baseAssetStatusUrl = SINGLE_SLASH + inventoryHostname + COLON + inventoryPortNumber + assetStatusUrlPart;        
        return success;
    }
    
    /**
     * test call of init() for inventory provider
     */
    public void testInit()
    {
        try
        {
            Registry.getRegistry().getInventoryProvider().init(
                    providerToConfig.get(Registry.getRegistry().getInventoryProviderPluginId()),
                    new TestTracer());
        }
        catch (Exception e)
        {
            fail("Exception in Init(): " + e.toString());
        }
    }
    
    /**
     * test get() of /asset/ to retrieve ids of all assets the provider knows about, in TDM Operational Model.
     */
    public void testGetAssetIds()
    {        
        try
        {
            IRestResponse restResponse = Registry.getRegistry().getInventoryProvider().get(new URI(baseAssetUrl));
            assertTrue(restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK);
            String response = restResponse.getXMLResponse();
            assertNotNull(response);
            Topology assets = TopologyUtils.parseTopologyFromXml(response);
            assertNotNull(assets);
            assertTrue(assets.getUnits().get(0) instanceof ServerUnit);
        }        
        catch (Exception e)
        {
            fail(e.toString());
        }
    }
    
    /**
     * test get() of /asset/* to retrieve a TDM model of all the assets the provider knows about. 
     */
    public void testGetAllAssets()
    {
        try
        {
            IRestResponse restResponse = Registry.getRegistry().getInventoryProvider().get(new URI(baseAssetUrl + "*"));
            assertTrue(restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK);
            String response = restResponse.getXMLResponse();
            assertNotNull(response);
            Topology assets = TopologyUtils.parseTopologyFromXml(response);
            assertNotNull(assets);
            assertTrue(assets.getUnits().get(0) instanceof ServerUnit);
        }        
        catch (Exception e)
        {
            fail(e.toString());
        }        
    }   
}
