/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
package com.ibm.rational.test.lm.spi.driver.test.util;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

import com.ibm.rational.test.lm.spi.IAutomationSPI;
import com.ibm.rational.test.lm.spi.IInventorySPI;
import com.ibm.rational.test.lm.spi.IVirtualizationSPI;

public class Registry 
{
    private static final String automationExtensionPoint = "com.ibm.rational.test.lm.spi.automation";
    private static final String inventoryExtensionPoint = "com.ibm.rational.test.lm.spi.inventory";
    private static final String virtualizationExtensionPoint = "com.ibm.rational.test.lm.spi.virtualization";
    private static final String clzz = "class";
    
    private static Registry registryInstance = new Registry();
    private IAutomationSPI automationProvider = null;
    private String automationProviderPluginId = "";    
    private IInventorySPI inventoryProvider = null;       
    private String inventoryProviderPluginId = "";
    private IVirtualizationSPI virtualizationProvider = null;
    private String virtualizationProviderPluginId = "";
    
    private Registry()
    {
        IExtensionRegistry registry = Platform.getExtensionRegistry();
        IConfigurationElement[] elements = registry.getConfigurationElementsFor(automationExtensionPoint);
        for (IConfigurationElement element : elements)
        {
            if (automationProvider == null)
            {
                try
                {
                    automationProvider = (IAutomationSPI)element.createExecutableExtension(clzz);
                    automationProviderPluginId = element.getAttribute("id");
                    System.out.println("Registering Automation Extension: " + automationProviderPluginId);                    
                }
                catch (Exception e)
                {
                    System.out.println("Error Registering Automation Extension " + element.getAttribute("id") + ": " + e.toString());
                }
            }
            else
            {
                System.out.println("More than one automation provider; skipping: " + element.getAttribute("id"));
            }
        }

        elements = registry.getConfigurationElementsFor(inventoryExtensionPoint);
        for (IConfigurationElement element : elements)
        {
            if (inventoryProvider == null)
            {
                try
                {
                    inventoryProvider = (IInventorySPI)element.createExecutableExtension(clzz);
                    inventoryProviderPluginId = element.getAttribute("id");
                    System.out.println("Registering Inventory Extension: " + inventoryProviderPluginId);
                }
                catch (Exception e)
                {
                    System.out.println("Error Registering Inventory Extension " + element.getAttribute("id") + ": " + e.toString());
                }
            }
            else
            {
                System.out.println("More than one inventory provider; skipping: " + element.getAttribute("id"));
            }
        }
        
        elements = registry.getConfigurationElementsFor(virtualizationExtensionPoint);
        for (IConfigurationElement element : elements)
        {
            if (virtualizationProvider == null)
            {
                try
                {
                	virtualizationProvider = (IVirtualizationSPI)element.createExecutableExtension(clzz);
                	virtualizationProviderPluginId = element.getAttribute("id");
                    System.out.println("Registering Virtualization Extension: " + virtualizationProviderPluginId);
                }
                catch (Exception e)
                {
                    System.out.println("Error Registering Virtualization Extension " + element.getAttribute("id") + ": " + e.toString());
                }
            }
            else
            {
                System.out.println("More than one virtualization provider; skipping: " + element.getAttribute("id"));
            }
        }
    }
    
    public static Registry getRegistry()
    {
        return registryInstance;
    }
    
    public IAutomationSPI getAutomationProvider()
    {        
        return automationProvider;
    }
    
    public String getAutomationProviderPluginId()
    {
        return automationProviderPluginId;
    }
    
    public IInventorySPI getInventoryProvider()
    {
        return inventoryProvider;
    }
    
    public String getInventoryProviderPluginId()
    {
        return inventoryProviderPluginId;
    }
    
    public IVirtualizationSPI getVirtualizationProvider() {
    	return virtualizationProvider;
    }
    
    public String getVirtualizationProvidePluginId() {
    	return virtualizationProviderPluginId;
    }
}
